;..............................................................................
;
;   disp_image 
;                 displays a two dimensional image
;
;                 Arguments
;
;                 cube = 2d-image
;                 x = 1d array containing x coords as a function of pixel #
;                 y = 1d array containing y coords as a function of pixel #
;		      (note uses fitaxes to produce these)
;
;		  (the maps are integrated over intervals of vdiff starting
;		   with the velocity vstart)
;               
;		  keywords
;
;		  levels = 1-d array containing levels for contouring
;                 xrange = range in x plotted
;                 yrange = range in y plotted
;                 percent = if set, levels are percent of max value
;                         (note max is the maximum of ALL channel maps)
;                 notv = if NOT set, plot image using the low and high values
;		       of the levels array as the mininum and maximum
;                 docontour = if set, plot contours
;                 nomatch = if set, requirement that aspect ratio of
;			    images must be conserved is relaxed
;		  nearest = if set, a nearest neighbour interpolation is
;		            used INSTEAD of a billinear interpolation
;                 ps = if set sent to postscript device
;                 label = [x,y] position, if set the maps 
;		           will be labeled with the velocity interval at
;		           this location
;                 continue = used for overlaying contours onto plots in
;			     postscript mode.  First plot image setting
;			     /tv, /nocontours, /ps and /continue, then
;			     plot contours setting only /ps.
;		  clip = clip data at given level (Kelvin)
;		  cosy = multiply the xaxis size by cos(midpt(y))
;		  cubic = use a cubic interpolation with parameter 0.5
;		  contour_color = color for contours
;
;
;   S. T. Megadeath - last updated 4-8-00
;   S. T. Megadeath - modified 7-30-2001
;   S. T. Megadeath - modified 10-16-2001
;----------------------------------------------------------------

pro disp_image,image,x,y,xrange=xrange,yrange=yrange,nomatch=nomatch,$
       docontours=docontours,notv=notv,ps=ps,levels=levels,percent=percent,$
       continue=continue, clip=clip, margin=margin,nomargin=nomargin,$
       xtitle=xtitle,ytitle=ytitle,nearest=nearest,cosy=cosy,loadcolor=loadcolor,$
       finish=finish,encapsulate=encapsulate,ct=ct,cubic=cubic,$
       contour_color=contour_color,color=color,noaxis=noaxis,$
       red=red,blue=blue,green=green,fxpos=fxpos,fypos=fypos,$
       noxaxis=noxaxis,noyaxis=noyaxis

;  set up initial values of margin size, cube size, and pixel increments

   xsize_cube = n_elements(x)
   ysize_cube = n_elements(y)
   onexpix = abs(x[1]-x[0])
   oneypix = abs(y[1]-y[0])

;  set position on display
   if NOT keyword_set(fxpos) then fxpos = [0,1]
   if NOT keyword_set(fypos) then fypos = [0,1]

;  load MINI colors if keyword is set
   if keyword_set(loadcolor) then begin 
      @loadcolor
   endif

;  if NOT keyword_set(color) then color = !d.table_size-1
;  set margins to 10% of paper size
   if NOT keyword_set(margin) then margin = 0.1

   if keyword_set(nomargin) then margin=0.

;  set nocontours
   if NOT keyword_set(docontours) then nocontours = 1

;  set contour_color
   if NOT keyword_set(contour_color) then contour_color=!d.table_size-1

   pos_array = replicate(0.,4)

; if keyword range isn't set, set the range to the maximum extent of the input data
   if keyword_set(xrange) then begin
      xsize = abs(xrange[1]-xrange[0])
   endif else begin
      xsize = n_elements(x)*onexpix
      xrange = [max(x)+onexpix/2.,min(x)-onexpix/2.]
   endelse  
   if keyword_set(yrange) then begin
      ysize = abs(yrange[1]-yrange[0])
   endif else begin
      ysize = n_elements(y)*oneypix
      yrange = [min(y)-oneypix/2.,max(y)+oneypix/2.]
   endelse

   if keyword_set(cosy) then begin
     ymid = (max(y)-min(y))/2.+min(y)
     xsize = xsize*cos(ymid*2*3.141592654/360.)
   endif

   im_asp = (1.*ysize)/xsize
   print,"xrange =",xrange,"yrange =",yrange

; set up postscript parameters
   if keyword_set(ps) then begin
     dpcm=1000.                      ; dots/cm
     xpaper=21.6                     ; xsize of paper (cm)
;     ypaper=27.9                     ; ysize of paper (cm)
     ypaper=21.6                     ; xsize of paper (cm)
     xmargin=2.5                     ; x-border (cm)
     ymargin=2.5                     ; y-border (cm)
     xoffset = xmargin/2.
     yoffset = ymargin/2.
     if NOT keyword_set(finish) then begin
        set_plot,'ps',/interpolate
       if NOT keyword_set(encapsulate) then begin
         device,/portrait,xoff=xoffset,yoff=yoffset,xsize=xpaper,ysize=ypaper,$
           filename="idl.ps",encapsulate=0
       endif else begin       
         device,/portrait,xoff=xoffset,yoff=yoffset,xsize=xpaper,ysize=ypaper,$
           filename="idl.ps",/encapsulate
       endelse
       device,/color,bits=8
       if NOT keyword_set(ct) then begin
         loadct,0
       endif else begin
         loadct,ct
       endelse
       if keyword_set(loadcolor) then begin
          @loadcolor
       endif
       if keyword_set(red) then tvlct,red,green,blue
       plot,x,y,xsty=5,ysty=5,/noerase,/nodata
     endif
   endif

;  set up initial value for display size, pixel size, and aspect ratio
   if keyword_set(ps) then begin
     dpcm = 1000.
     xdisp = (xpaper-xmargin)
     ydisp = ypaper-ymargin
     disp_asp =  ydisp/xdisp
   endif else begin
     dpcm = 1.
     xdisp = !D.x_vsize
     ydisp = !D.y_vsize
   endelse

; take into account a fractional offseting in image display
     xdisp = xdisp*(fxpos[1]-fxpos[0])
     ydisp = ydisp*(fypos[1]-fypos[0])
     disp_asp = 1.*ydisp/xdisp  

; set aspect ratio
   if NOT keyword_set (nomatch) then begin
      aspect = im_asp
     endif else begin
      aspect = disp_asp
   endelse

; set size of plots and margin of plot grid
    if (disp_asp gt aspect) then begin
      xpsize = ((1.-margin)*xdisp)
      ypsize = (aspect*xpsize)
      marginx = (margin/2.*xdisp)+xdisp*fxpos[0]/(fxpos[1]-fxpos[0])
;      marginy = (marginx*disp_asp)
       marginy = (ydisp-ypsize)/2.+ydisp*fypos[0]/(fypos[1]-fypos[0])
      print,"X",xpsize,ypsize
    endif else begin
      ypsize = ((1.-margin)*ydisp)
      xpsize = (ypsize/aspect)
      marginy = (margin/2.*ydisp)+ydisp*fypos[0]/(fypos[1]-fypos[0])
;      marginx = (marginy/disp_asp)
      marginx = (xdisp-xpsize)/2.+xdisp*fxpos[0]/(fxpos[1]-fxpos[0])
      print,"Y",xpsize,ypsize,marginy,marginy+ypsize,ydisp
    endelse

; create a position array of the corners of each plot panel
   pos_array[*] = [marginx,marginy,marginx+xpsize,marginy+ypsize]
   if keyword_set(ps) then begin
     pos_array = pos_array*dpcm
   endif else begin
     pos_array = round(pos_array)
   endelse

   print,pos_array

; find maximum value of all the channel maps
   maxval = max(image)

   print,"maximum value of image =",maxval

; if the level keyword isn't set, then set levels of 30 to 100% of the
; maximum value
   if NOT keyword_set (levels) then begin
     levels = maxval*(findgen(7)+3)/10.
   endif

; if precent level keyword is set, scale levels to precentage of max value
   if keyword_set (percent) then begin
     levels = levels*maxval/100.
   endif


; if tv keyword is set, interpolate data onto new grid and plot image
;     if NOT keyword_set(notv) then begin
       if NOT keyword_set(ps) then begin
         nxpix = pos_array[2]-pos_array[0]
         nypix = pos_array[3]-pos_array[1]
       endif else begin
         nxpix = 800
         nypix = 800*aspect
       endelse
       xnew = $
         findgen(nxpix+1)*(xrange[1]-xrange[0])/float(nxpix)+xrange[0]
       ynew = $
         findgen(nypix+1)*(yrange[1]-yrange[0])/float(nypix)+yrange[0]
       xinterp = interp_pix(xnew,x)
       yinterp = interp_pix(ynew,y)
       if keyword_set(nearest) then begin
          xinterp = round(xinterp)
          yinterp = round(yinterp)
       endif
if keyword_set(cubic) then begin
       plotmap = interpolate(image,xinterp,yinterp,cubic=-.5,/grid,missing=0)
endif else begin
       plotmap = interpolate(image,xinterp,yinterp,/grid,missing=0)
endelse
     if NOT keyword_set(notv) then begin
       tv,bytscl(plotmap,min=levels[0],max=levels[n_elements(levels)-1],$
          top = !d.table_size-2),$
          pos_array[0],pos_array[1],xsize=xpsize*dpcm,ysize=ypsize*dpcm
     endif

; plot contour map
;     contour,min_curve_surf(image,xvalues=x,yvalues=y,nx=n_elements(x),ny=n_elements(y)),x,y,xtickformat='(" ",a1)',ytickformat='(" ",a1)',$
;     contour,image,x,y,xtickformat='(" ",a1)',ytickformat='(" ",a1)',$
      contour,plotmap,xnew,ynew,xtickformat='(" ",a1)',ytickformat='(" ",a1)',$
            xstyle=1,ystyle=1,$
            position = pos_array[*],/noerase,$
            xrange = xrange, yrange = yrange,$
            levels = levels,/device,nodata=nocontours,$
            c_color=contour_color,color=color

; label each plot if keyword label is set
    if keyword_set(label) then begin
      xy = convert_coord(label[0],label[1],/data,/to_device)
      vel1 = vstart+i*vdiff
      vel2 = vstart+(i+1)*vdiff
      strlabel=strcompress(string(vel1)+","+string(vel2))
      xyouts,xy[0],xy[1],strlabel,color=color,/device
    endif

; plot contour map without data to set up axis values.
    if keyword_set(noyaxis) then begin
     contour,image,x,y,/nodata,ytickformat='(" ",a1)',$             
             xstyle=1,ystyle=1,$
             position = pos_array,/noerase,$
             xrange = xrange, yrange = yrange,/device,$
             color=color,xtitle=xtitle,ytitle=ytitle,$
	     charsize=1.
    endif


; plot contour map without data to set up axis values.
    if keyword_set(noxaxis) then begin
     contour,image,x,y,/nodata,xtickformat='(" ",a1)',$             
             xstyle=1,ystyle=1,$
             position = pos_array,/noerase,$
             xrange = xrange, yrange = yrange,/device,$
             color=color,xtitle=xtitle,ytitle=ytitle,$
	     charsize=1.
    endif

; plot contour map without data to set up axis values.
    if (NOT keyword_set(noaxis) and NOT keyword_set(noxaxis) $
	and NOT keyword_set(noyaxis))then begin
     contour,image,x,y,/nodata,$
             xstyle=1,ystyle=1,$
             position = pos_array,/noerase,$
             xrange = xrange, yrange = yrange,/device,$
             color=color,xtitle=xtitle,ytitle=ytitle,$
	     charsize=1.
    endif

; if making postscript file, close device and change plot back to 'x'
   if (keyword_set(ps) and NOT keyword_set(continue)) then begin
      device,/close
      set_plot,'x',/interpolate
   endif

   return
   end

   

