;+
; NAME:
;       PHOTMATCH3
;
; PURPOSE:
;       Match detections in multiple bands from PhotVis output files
;
; CALLING SEQUENCE:
;       result = PHOTMATCH3()
;
; INPUTS:
;       NONE (GUI-based)
;
; OPTIONAL INPUTS:
;       NONE
;
; KEYWORDS:
;       TOL - distance tolerance in pixels for detection match
;       OFFSET - 2*(n_bands-1) element vector of offsets between source images
;                from the first band
;       FILTER - string array with filter names of data to be matched
;
; OUTPUTS:
;       result is an n_bands+2 by n_matches element array 
;         [x,y,mag1,mag2(,mag3)]
;
; PROCEDURE CALLS:
;       fits_read, fits_write, fits_open, fits_close, sxpar()
;
; REVISION HISTORY:
;       07-01-02 - initial coding (RAG)
;       11-4-02: bastardized from photmatch (RAG)
;
;-


function photmatch3,tol=tol,offset=offset,filter=filter

 if not keyword_set(tol) then tol = 20.
 if not keyword_set(filter) then filter = ['base','new']
 nf = n_elements(filter)
 if not keyword_set(offset) then offset = fltarr(2*(nf-1))

 data = ptrarr(nf,/alloc)

 for i=0,nf-1 do begin
  name = dialog_pickfile(title='Select '+filter[i]+' PV_DAT file',$
                             /must_exist)
  if name eq '' then begin
   print,'Cancelled...'
   retall
  endif
  restore,name
  if i gt 0 then begin
   pv_dat[0,*] = pv_dat[0,*] + offset[2*(i-1)]
   pv_dat[1,*] = pv_dat[1,*] + offset[1+2*(i-1)]
  endif
  *data[i] = temporary(pv_dat)
 endfor

 name = dialog_pickfile(title='Select source FITS image',$
                             /must_exist)
 if name eq '' then begin
  print,'Cancelled...'
  retall
 endif

 fits_read,name,im,hdr
 im = rebin(im,(size(im))[1]*2,(size(im))[2]*2)

 sz = size(*data[0])
 k = 0

 window,0,xs=sxpar(hdr,'naxis1')*2,ys=sxpar(hdr,'naxis2')*2

 for i = 0,sz[2]-1 do begin
  tv,bytscl(im,min=median(im),max=median(im)+300)
  plots,(*data[0])[0,*]*2,(*data[0])[1,*]*2,/dev,color=255l,psym=7,symsize=.7

  x = (*data[0])[0,i]
  y = (*data[0])[1,i]
  mag = (*data[0])[7,i]
  gdflg = 0
  tvcircle,tol*2,x*2,y*2,255l

  j = 1
  dist = sqrt(((*data[j])[0,*]-x)^2+((*data[j])[1,*]-y)^2)
  w = where(dist lt tol,nt)
  if nt gt 0 then begin

   plots,(*data[1])[0,w]*2,(*data[1])[1,w]*2,/dev,color=255l*256l,psym=2,symsize=.7
   xyouts,(*data[1])[0,w]*2+2,(*data[1])[1,w]*2+2,strtrim(indgen(nt),2),/dev,color=255l*256l
   temp = fltarr(4,nt)
   temp[0,*] = findgen(nt)
   temp[1:2,*] = (*data[j])[0:1,w]
   temp[3,*] = (*data[j])[7,w]
   print,temp
   print,'Matching object index? (hit Enter for none)'
   ind = ''
   read,ind
   if strlen(ind) gt 0 then begin
    new = fix(ind)
    outnew = fltarr(3*nf+1)
    outnew[0]=i&outnew[1]=x&outnew[2]=y&outnew[3]=mag
    outnew[4:5]=(*data[j])[0:1,w[new]]&outnew[6]=(*data[j])[7,w[new]]
    if k eq 0 then begin
     out = outnew
    endif else begin
     temp1 = temporary(out)
     out = fltarr(3*nf+1,k+1,/nozero)
     out[0,0] = temporary(temp1)
     out[0,k] = outnew
    endelse
    k = k+1
   endif else print,'no match'
  endif else print,'no match'
 endfor

 return,out

end

