;+
; NAME:
;   WCS_PHOTMATCH
;
; PURPOSE:
;   Match detections in multiple bands from WCS-calibrated PhotVis output files
;
; CALLING SEQUENCE:
;   result = WCS_PHOTMATCH()
;
; INPUTS:
;   NONE (GUI-based)
;
; OPTIONAL INPUTS:
;   NONE
;
; KEYWORDS:
;   TOL - distance tolerance in arcsec for detection match. default is 1.
;   BANDS - string array with band names of data to be matched ( defaults to
;           ['J','H','K','1','2','3','4'] for near IR and IRAC merger)
;   NO_NEAR - set this switch to make pm_filter.pro remove all detections with
;             other detections within tol pixels from each pv_dat before trying
;             to match...
;   AVGPOS - set to return the mean position for each match instead of
;            all positions.
;   WRITE - set to get prompted for filename to write output to ASCII table
;
; OUTPUTS:
;   result is a 4*n_bandss x m_matches array (unless avgpos is set):
;       [x1,y1,...,xn,yn,mag1,...,magn,magerr1,...,magerrn]
;
; PROCEDURE CALLS:
;   pm_filter(),pm_noise()
;
; REVISION HISTORY:
;   07-01-02: initial coding (RAG)
;   07-09-03: added NO_NEAR keyword (RAG)
;   11-03-03: coverted for IRAC data, allowed for objects than have
;             less than all matches (search for -100 mag), and changed
;             output to dump ALL positions, with avg position still
;             accessible by AVGPOS. (RAG)
;   02-21-04: bring in WCS, tol in arcsec now (RAG)
;   03-01-04: updated docs, cleaned up and tested for public distribution. (RAG)
;   03-09-04: sloppy hack to output photometric uncertainties...  Will be 
;             optional someday. (RAG)  
;
;-


function wcs_photmatch,tol=tol,bands=bands,no_near=no_near,avgpos=avgpos,$
                       write=write,error_cap=error_cap

 if not keyword_set(tol) then tol = 1.
 if not keyword_set(bands) then bands = ['J','H','K','1','2','3','4']
 nf = n_elements(bands)

 tol2 = tol/3600.

 data = ptrarr(nf,/alloc)
 wp = ptrarr(nf,/alloc)

 for i=0,nf-1 do begin
  name = dialog_pickfile(title='Select '+bands[i]+' band PV_DAT file',$
                             /must_exist)
  if name eq '' then begin
   print,'Cancelled...'
   retall
  endif
  restore,name
  *data[i] = temporary(pv_dat)
 endfor

 for i=0,nf-1 do begin
  if keyword_set(no_near) then *data[i] = pm_filter(temporary(*data[i]),tol,/wcs)
  if keyword_set(error_cap) then *data[i] = pm_noise(temporary(*data[i]),error_cap[i])
  sz = size(*data[i])
  *wp[i] = lindgen(sz[2])
 endfor

 if keyword_set(write) then begin
  write = dialog_pickfile(title='Save output ASCII table as')
  if write eq '' then delvarx,write
 endif

 if not keyword_set(write) then print,'Not writing output to ASCII file'

 g = 0l

 for j = 0,nf-1 do begin
  sz = size(*data[j])

  for i = 0l,sz[2]-1 do begin

   if (*wp[j])[0] ne -1 then begin

    wt = where(*wp[j] ne i,nct)
    if nct lt n_elements(*wp[j]) then begin
     if nct gt 0 then *wp[j] = (*wp[j])[wt] else *wp[j] = [-1]
     x = (*data[j])[0,i]
     y = (*data[j])[1,i]

     gdflg = intarr(nf) & gdflg[j]=1;0
     xtmp = dblarr(nf) & ytmp = dblarr(nf) & mags = dblarr(nf)-100. & errors=dblarr(nf)+10.
     xtmp[j] = x  & ytmp[j] = y & mags[j] = (*data[j])[7,i] & errors[j] = (*data[j])[8,i]

     for k = 0,nf-1 do begin
      if k ne j and (*wp[k])[0] ne -1 then begin
       dist = sqrt((((*data[k])[0,*wp[k]]-x)*$
          cos(!pi/180.*(*data[k])[1,*wp[k]]))^2+((*data[k])[1,*wp[k]]-y)^2)
       md = min(dist)
       if md lt tol2 then begin ;gdflg = gdflg+1
        w = (where(dist eq md))[0]
        wn = where(dist ne md,nn)
        xtmp[k] = (*data[k])[0,(*wp[k])[w]]
        ytmp[k] = (*data[k])[1,(*wp[k])[w]]
        mags[k] = (*data[k])[7,(*wp[k])[w]]
        errors[k] = (*data[k])[8,(*wp[k])[w]]
        if nn gt 0 then *wp[k] = (*wp[k])[wn] else (*wp[k]) = [-1]
        gdflg[k] = 1
       endif
      endif
     endfor

     w = where(gdflg eq 1,ngd)
     if keyword_set(avgpos) then begin
      x = mean(xtmp[w])
      y = mean(ytmp[w])
      xy = [x,y]
      nxy = 2
     endif else begin
      nxy = 2*nf
      xy = dblarr(2*nf)
      for c = 0l,ngd-1 do xy[0+2*w[c]:1+2*w[c]] = [xtmp[w[c]],ytmp[w[c]]]
     endelse

     if g eq 0 then out = [xy,mags,errors] else begin
      temp = temporary(out)
      out = dblarr(2*nf+nxy,g+1,/nozero)
      out[*,0:g-1] = temporary(temp)
      out[*,g] = [xy,mags,errors]
     endelse

     g = g+1l
    endif
   endif
  endfor
 endfor

 if keyword_set(write) then begin
  m_matches=n_elements(out[0,*])
  print,'Writing output to ASCII table '+write
  openw,one,write,/get
  title = ['RA_'+bands[0],'Dec_'+bands[0]]
  title2 = 'err_'+bands
  for j=1,nf-1 do title = [title,'RA_'+bands[j],'Dec_'+bands[j]]
  title = [title,bands,title2]
  printf,one,strjoin(title,' ',/single)
  for i=0l,m_matches-1 do printf,one,strjoin(strtrim(out[*,i],2),' ',/single)
  free_lun,one
 endif
  
 return,out

end

