;+
;   name:
;        assemble.pro
;
;   purpose:
;       this function produces a list of coordinates of offsets for stars
;       in a master and slave image; also outputs the final mosaic and
;       the mask for that mosaic.
;
;   calling sequence:
;       result = assemble(mastername,slavelist,mastermaskname,slavemasks,
;           coadd=coadd)
;
;   inputs:
;       mastername = master image
;       slavelist = array of slave images
;       mastermaskname = fits file name of mask for the master image
;             (Obsolete)
;       slavemasks = array of mask images for the corresponding slaves
;             (Obsolete)
;
;   keywords:
;       coadd = set to use coadd mode (additional automation)
;       scl = used in rob_assemble.pro front-end
;       zflg = used in rob_assemble.pro front-end
;
;   output:
;       result = structure including the 'mosaic', mask for the mosaic
;                ('master'), xoffsets ('xoff'), yoffsets ('yoff')
;
;   called programs:
;       fits_datmask.pro
;       loadlist.pro
;       rob_reg.pro
;       mosaic_two.pro
;
;   history:
;       written by S. T. Megeath
;       7-??-00 - first version released STM)
;       2-26-01 - program modified to return structure containing
;                 mosaic and offsets (STM)
;       3-20-01 - set autoffset keyword for mosaic_two (STM)
;       3-20-01 - now uses "loadlist" to load in list of slave images,
;                 nslaves parameter eliminated (STM)
;       4-23-01 - program now reads in an array of masks for
;                 corresponding slaves instead of
;                 just one mask for all slaves (DEP)
;       6-8-01  - replaced doregister.pro usage with rob_reg.pro (RAG)
;       6-12-01 - added ability to go back one iteration to allow attempt
;                 to redo registration if rob_reg.pro screws up.
;                 also added coadd keyword (RAG)
;       6-15-01 - killed tvmax and tvmin in favor of mean +/- 1 sigma
;                 to scale master display and killed xpos and ypos because
;                 I kept getting random errors concerning them if they
;                 weren't specified (RAG)
;       7-2-01  - made minor changes to accommodate "persistent scaling"
;                 feature added to rob_reg.pro and rob_reg_gui.pro (RAG)
;       9-5-01  - switched to fits_datmask for mask managenment, thus redering
;                 mastermaskname and slavemasks obsolete. (RAG)
;       6-10-02 - switched weight plane to be mask extension too (RAG)
;       3-10-03 - added zflg transfer keywd for rob_assemble improvements (RAG)
;	7-10-03 - corrected offset problem
;-

function assemble,mastername,slavelist,mastermaskname,slavemasks,$
         coadd=coadd,scl=scl,zflg=zflg,moff=moff

; load in master and mask file
;mod-RAG- switched to fits_datmask for mask management
;  fits_read,mastername,master
;  fits_read,mastermaskname,mastermask
 if not keyword_set(coadd) then coadd = 0
 fits_datmask,mastername,master,hdr,mvect

 msz = size(mvect)
 if msz[0] lt 2 then begin
  mastermask = master * 0 + 1 & mastermask[mvect] = 0
  master[mvect] = median(master)
 endif else begin
  mastermask = mvect
 endelse

; load in slave files and their masks (should be same number)
  slavenames = loadlist(slavelist)
;  slavemasknames = loadlist(slavemasks)
  nslave = n_elements(slavenames)

; initialize offsets and master offsets
  offset = fltarr(2,10000)
  offset[*,0] = [0.,0.]
  master_xoff = 0.
  master_yoff = 0.

  for i=0,nslave-1 do begin
;mod-RAG- switched to fits_datmask for mask management
;    fits_read,slavenames[i],slave
;    fits_read,slavemasknames[i],mask
   fits_datmask,slavenames[i],slave,hdr,mvect

   msz = size(mvect)
   if msz[0] lt 2 then begin
    mask = slave * 0 + 1 & mask[mvect] = 0
    slave[mvect]=median(slave)
   endif else begin
    mask = mvect
   endelse

; after first iteration, initialize redo button in GUI call
    if i gt 0 then begin
     sw1 = 1
     if not keyword_set(coadd) then sw2 = 0
    endif else begin
     sw1 = 0 & sw2 = 0
    endelse
    out = rob_reg(master,slave,redo=sw1,coadd=sw2,scl=scl)

; if out is a string, then REDO button was hit,
;  and therefore we need to go back to previous iteration!
    so = size(out)
    if so[so[0]+1] eq 7 then begin
     master = temporary(om) & slave = temporary(os)
     mastermask = temporary(omm) & mask = temporary(osm)
     out = rob_reg(master,slave,scl=scl)
     i = i - 1

;STM reset master offset
     master_xoff = master_xoff-xmoff
     master_yoff = master_yoff-ymoff
    endif

; remember this iteration so we can go back if need be
    om = master & os = slave
    omm = mastermask & osm = mask

; format output from rob_reg.pro to fit existing call to mosaic_two.pro
    xoff = out[0] & yoff = out[1] & zflg = out[2]

;STM write offsets to file - these offsets are relative to the
; the original master.
    offset[0,i+1] = xoff-master_xoff
    offset[1,i+1] = yoff-master_yoff

;RAG - try and pray
    txoff = xoff & tyoff = yoff
    irac_spoffset,slave,hdr,mask,kx,ky,txoff,tyoff
    slave = poly_2d(temporary(slave)*mask,kx,ky,1,missing=0)
    mask = poly_2d(temporary(mask),kx,ky,1,missing=0)

    delvarx,kx,ky

    wbd = where(mask lt .1,nbd)
    wgd = where(mask ge .1,ngd2)
    if nbd gt 0 then begin
     slave[wbd] = 0.
     mask[wbd] = 0.
    endif
    if ngd2 gt 0 then slave[wgd] = slave[wgd]/mask[wgd]

; mosaic new stuff
    mosaic_two,master,mastermask,slave,mask,txoff,tyoff,xmoff,ymoff,/autoffset

; mosaic
;    mosaic_two,master,mastermask,slave,mask,xoff,yoff,xmoff,ymoff,/autoffset

;RAG endprayer

;STM track how much the master has been offset during the assembly
; process
    master_xoff = master_xoff+xmoff
    master_yoff = master_yoff+ymoff
    print,offset[*,0:i+1],' offset'
  endfor

; open master window and fit data onto mosaic display
  xmaster = n_elements(master[*,0])/fix(zflg)
  ymaster = n_elements(master[0,*])/fix(zflg)
  xslave = n_elements(slave[*,0])/fix(zflg)
  yslave = n_elements(slave[0,*])/fix(zflg)
  
;mod-RAG- using scaling as user sets in rob_reg_gui.pro.  cool, eh?
  if keyword_set(coadd) then begin
   window,5,xs=2*xmaster,ys=ymaster,retain=2,$
      title="Did things register properly?"
   tvmaster = bytarr(2*xmaster,ymaster)
   tvmaster[0:xslave-1,0:yslave-1] = bytscl(congrid(slave,xslave,yslave),$
      min=scl[1,0],max=scl[1,1])
   tvmaster[xmaster:2*xmaster-1,*] = bytscl(congrid(master,xmaster,ymaster),$
      min=scl[0,0],max=scl[0,1])
  endif else begin
   window,5,xs=xmaster,ys=ymaster,retain=2,title="Master"
   tvmaster = bytscl(congrid(master,xmaster,ymaster),min=scl[0,0],max=scl[0,1])
  endelse 
  tv,tvmaster

; return mosaic, mosaic mask, and offsets as a structure
;  file_delete,'~/anger.tmp'
  moff = [master_xoff,master_yoff]
  return,{mosaic:master,master:mastermask,$
    xoff:offset[0,0:nslave],yoff:offset[1,0:nslave]}

end
