;+
; NAME:
;       FITS_DATMASK
;
; PURPOSE:
;       Manage FITS format data with mask vector as FITS extension
;
; CALLING SEQUENCE:
;       FITS_DATMASK, inname [, im, hdr, mask ]
;
; INPUTS:
;       inname - string with file name for reading or writing
;
; OPTIONAL INPUTS:
;       im - 2D image (data)
;       hdr - string array (FITS standard header)
;       mask - 2D binary image (mask) or vector of bad pixels
;
; KEYWORDS:
;       WRITE - set to write input data to file (default is read from file)
;       NO_VECT - set to return mask as a 2D binary image (default is to
;                 return vector of bad pixels)
;
; OPTIONAL OUTPUTS:
;       im - 2D image (data)
;       hdr - string array (FITS standard header)
;       mask - 2D binary image (mask) or vector of bad pixels (if mask
;              doesn't exist, mask is defined as -1)
;
; PROCEDURE CALLS:
;       fits_read, fits_write, fits_open, fits_close, sxpar()
;
; REVISION HISTORY:
;       8/23/01 - initial coding (RAG)
;       5/13/02 - rehashed to allow compatibility with regular fits_read (RAG)
;       6/10/02 - another rehash to allow use of vector _or_ weight plane in
;                 badpix extension (RAG)
;       2/4/03 - weird behavior of EXTEND tag in FITS header was due to my
;                accidental reliance on an ancient version of fits_read.pro...
;                got myself a real version and fixed this to make them work
;                together properly. (RAG)
;       11/10/03 - fixed subtle bug with no_vect mask conversion. (RAG)
;
;-

pro fits_datmask,inname,im,hdr,mask,write=write,no_vect=no_vect

;open inname and get fits control block
 fits_open,inname,fcb,write=write

;read the data and hdr if in read mode
 if not keyword_set(write) then begin
  fits_read,fcb,im,hdr

;check for the mask extension, and read that too if it exists
  if sxpar(hdr,'EXTEND') then begin
   fits_read,fcb,mask,xtension='badpix'
  endif else mask = -1
 endif

;figure out qualities of mask
 sm = size(mask)

 case sm[0] of
  2: begin
   n0 = n_elements(where(mask eq 0))+n_elements(where(mask eq 1))
   nmask = n_elements(mask)
   if n0 eq nmask then begin
    if not keyword_set(no_vect) then begin
     mask = where(temporary(mask) eq 0)
    endif
   endif
  end
  1: begin
   if keyword_set(no_vect) then begin
    temp = im * 0 + 1
    duh = where(mask ge 0,dct)
    if dct gt 0 then temp[mask] = 0
    mask = temporary(temp)
   endif
  end
  else: break
 endcase

;figure out qualities of mask again, because it might have changed
 sm = size(mask)

;write data, hdr, (and mask, if it exists) to file
 if keyword_set(write) then begin
  fits_write,fcb,im,hdr
  if sm[0] gt 0 then fits_write,fcb,mask,xtension='badpix'
 endif

;close the fits control block
 fits_close,fcb

end
