;+
;   name:
;       mosaic_two.pro
;
;   purpose:
;       mosaicing subroutine for interactive mosaic construction
;
;   calling sequence:
;       mosaic_two, mosaic, mosmask, data, datamask, xoff, yoff,
;       xmoff, ymoff, autoffset=autoffset
;
;   inputs:
;       mosaic = mosaic image which should be modified
;       mosmask = mask accompanying mosaic data
;       data = data array to be added to mosaic
;	datamask = mask for data
;       xoff,yoff = x and y offsets of data-these values will
;				 be modified and returned by mosaic_two
;       xmoff,ymoff = x and y offsets of initial mosaic - these
;				 values are returned so that the offsets
;				 of the previously mosaicked images
;				 can be registered.
;       autoffset:   set on to automatically adjust median levels
;				 between frames
;   output:
;       result is an image mosaic
;
;   called programs:
;       interp.pro
; 
;   history:
;       7-20-00  - first version released, written by S. T. Megeath
;       3-20-01  - added autoffset keyword (STM)
;       4-11-01  - (STM and DEP) fixed lines 74, 75 and 76 by using values
;                 of .99 instead of .9 and .01 instead of .1 as conditions.
;                 before the change, some images weren't aligning correctly
;                 because they fell under the .1 threshold.  added lines
;                 77-78.
;       4-23-01  - DEP took out lines 77-78 and added an else
;                  statement to the if in lines 75 and 76.
;       6-21-01  - changed function of autoffset to offset by difference
;                  in median values of pre-expansion mosaic and image being
;                  added.  (RAG)
;       7-17-01  - changed some of the calcs to use the temporary function
;                  to use memory more efficiently (RAG)
;
;-

pro mosaic_two,mosaic,mosmask,data,datamask,xoff,yoff,xmoff,ymoff,$
	autoffset=autoffset

;  medmo = median(mosaic)
;  medim = median(data)

;  print,'medians: ',medmo,medim

; find array sizes
  xmos = n_elements(mosaic[*,0])
  ymos = n_elements(mosaic[0,*])
  xim  = n_elements(data[*,0])+2
  yim  = n_elements(data[0,*])+2

; expand image
  im = replicate(0.,xim,yim)
  mask = replicate(0.,xim,yim)
  im[1:xim-2,1:yim-2] = data  
  mask[1:xim-2,1:yim-2] = datamask
  xoff = xoff-1
  yoff = yoff-1.
print,xoff,yoff

; find edges of data and necessary mosaic size 
  rmos = xmos > (fix(xoff)+xim+1)
print,rmos
  lmos = 0 < fix(xoff)-1
  umos = ymos > (fix(yoff)+yim+1)
  dmos = 0 < fix(yoff)-1
  mosxsiz = rmos-lmos+1
  mosysiz = umos-dmos+1
  print,'mosaic edges',xmos,ymos,fix(yoff)+yim+1,rmos,lmos,umos,dmos

; create newmosaic and load mosaic with original mosaic
  newmosaic = replicate(0.0,mosxsiz,mosysiz)
  newmosmask = replicate(0.0,mosxsiz,mosysiz)
  newmosaic[-lmos:xmos-lmos-1,-dmos:ymos-dmos-1] = mosaic
  newmosmask[-lmos:xmos-lmos-1,-dmos:ymos-dmos-1] = mosmask
  mosaic = newmosaic
  mosmask = newmosmask
  help,mosaic
  xmoff = -lmos
  ymoff = -dmos
  xoff = xoff-lmos
  yoff = yoff-dmos

; calculate offsets in integer pixels and fractional pixels
  print,xoff,yoff
  x0 = fix(xoff+.99) & y0 = fix(yoff+.99) & xlen = xim & ylen = yim
  if (xoff-1.*fix(xoff) gt .01) then xoff = x0*1.-xoff else xoff=0.
  if (yoff-1.*fix(yoff) gt .01) then yoff = y0*1.-yoff else yoff=0.
;  if (xoff-1.*fix(xoff) le .01) then xoff = 0.
;  if (yoff-1.*fix(yoff) le .01) then yoff = 0.
  print,x0,xoff,y0,yoff

; billinear interpolation of data and mask 
  in_im = im*mask
  in_mask = mask
  im = interp(in_im,xoff,yoff)
  mask = interp(in_mask,xoff,yoff)

; calculate median of difference 
  if keyword_set(autoffset) then begin
    off_index = where(mosmask[x0:x0+xlen-1,y0:y0+ylen-1] gt .5 and $
       mask gt .5 and autoffset eq 1)
    if(n_elements(off_index) gt 10) then begin
;       offset = im-mosaic[x0:x0+xlen-1,y0:y0+ylen-1]
;       off_median = median(offset(off_index))
;       off_median = medim-medmo 
       tmp_mos = mosaic[x0:x0+xlen-1,y0:y0+ylen-1]
       off_median = median(im[off_index]/mask[off_index]-tmp_mos[off_index])
    endif else begin
       off_median = 0.0
    endelse
  endif else begin
    off_median = 0.0
  endelse

  print,'off_median = '+strtrim(off_median,2)

; add mask weighted data into mosaic array 

  mosaic = temporary(mosaic)*mosmask
  mosaic[x0:x0+xlen-1,y0:y0+ylen-1]=$
      temporary(mosaic[x0:x0+xlen-1,y0:y0+ylen-1])+$
      (im-off_median*mask)

; add mask into mask array 

  mosmask[x0:x0+xlen-1,y0:y0+ylen-1]=$
      temporary(mosmask[x0:x0+xlen-1,y0:y0+ylen-1])+mask

; divide data by mask

  index = where(mosmask gt 0.1)
  index_not = where(mosmask le 0.1,nin)
  if(index[0] eq -1) then begin
      mosaic = temporary(mosaic)/mosmask
  endif else begin
      mosaic[index] = temporary(mosaic[index])/mosmask[index]
      if nin gt 0 then mosmask[index_not] = 0. 
  endelse       

end

