;------------------------------------------------------------- 
;
;  convert_map - converts coordinate system of an image by rebinning
;		 the image so that each pixel is preplaced by four
;		 smaller pixels, and then using the nearest neigbhour
;		 technique for regridding the points onto a new 
;		 coordinate system.  The rotation point of the maps
;		 is the center pixel (defined by fix(npix/2).
;		 This program uses the "jprecess", "bprecess",
;		 and "euler" functions of the idl astronomy package.
;        
;  !warning - the conversion does not guarantee conservation of
;	      flux in an image
;
;  parameters: data - 2d image
;	       x - 1d vector containing the coordinate for each
;		   row in data
;	       y - 1d vector containing the coordinate for each
;		   column in data
;	       conversion - a number from 1-6
;			    1 - precess from 1950 -> 2000
;			    2 - precess from 2000 -> 1950
;			    3 - convert from galactic -> 2000
;			    4 - convert from galactic -> 1950
;			    5 - convert from 2000 -> galactic
;			    6 - convert from 1950 -> galactic
;
;
;  Primary Author: S. T. Megeath
;
;  History:
;
;  5-9-2000 - 1st version released
;
;--------------------------------------------------------------

function convert_map,data,x,y,conversion

; find size of data
  xsize = n_elements(x)
  ysize = n_elements(y)

; mid point of data
  x_pix = fix(xsize/2.)
  y_pix = fix(ysize/2.)
  convert,x[x_pix],y[y_pix],xconv_center,yconv_center,conversion

; find spacing of data
  deltax = x[0]-x[1]
  deltay = y[0]-y[1]

; find rotation between new data and old data
  convert,x[x_pix],y[y_pix]+deltay,xconv_rot,yconv_rot,conversion
  theta = atan((xconv_rot-xconv_center)/(yconv_rot-yconv_center))

; expand size of returned image to encompass the old data
  yconv_size = abs(cos(theta)*ysize)+abs(xsize*sin(theta))
  xconv_size = abs(cos(theta)*xsize)+abs(ysize*sin(theta))*8.
  xconv_pix = fix(xconv_size/2.)
  yconv_pix = fix(yconv_size/2.)

; create xnew and ynew
  xnew = (findgen(xconv_size)-xconv_pix)*deltax+xconv_center
  ynew = (findgen(yconv_size)-yconv_pix)*deltay+yconv_center

; convert data
  image = convert_nearest(data,x,y,xnew,ynew,conversion)

; create structure with old and new data  
  conv = {image:image, x:xnew,y:ynew}

  return,conv

end
