;
; gather_2mass program
;
;	PARAMETERS
;
;	xoutsize - number of samples in RA dimension
;	youtsize - number of samples in Dec dimension
;	scale   - sample spacing in arcsecs
;
;	KEYWORDS
;	 
;	infile = 2MASS PSC ASCII table filename, or 5 x nstars array  
;	write = switch or filename to write FITS image of output to 
;               (INCLUDES WCS DATA INSERTED IN HEADER!!)
;	outwcs - outputs structure containing wcs info of gather plot
;	numstars - the number of stars to gather (can be a vector, if you'd 
;                  like multiple maps as output) (default = 7)
;	maglimit - choose stars brigther than maglimit (default = 100)
;	dist     - set to distance in pc (default = 1)
;
; S. T. Megeath 7-12-2003
; R. A. Gutermuth 8-18-03
; R. A. Gutermuth 8-27-03
; 10-15-03: cleaned up and modified gather_pv3.pro for use with 2MASS 
;           ASCII tables.  (RAG)
;
;-----------------------------------------------------------------
 
function gather_2mass,xoutsize,youtsize,scale,infile=infile,write=write,$
	        maglimit=maglimit,numstars=numstars,dist=dist,outwcs=outwcs

; set keyword
  if NOT keyword_set(maglimit) then maglimit = 100
  if NOT keyword_set(numstars) then numstars = 7
  if NOT keyword_set(dist) then dist = 1
  areaconv = 1
  nlayers = n_elements(numstars)

  if keyword_set(infile) then begin
   if size(infile,/type) eq 7 then indat = parse_2mass4(infile=infile) else $
      indat = infile
  endif else indat = parse_2mass4()

; find all stars with magnitudes less than our equal to the mag limit and
; store their xpositions and ypositions

  index = where(indat[4,*] le maglimit,nstars)
  if nstars lt min(numstars) then begin
   print,'ERROR in gather_2mass.pro: not enough valid sources!!'
   retall
  endif

  med_radec = [double(median(indat[0,index])),double(median(indat[1,index]))]
  med_xy = fix([xoutsize,youtsize])/2
  xpos = (med_radec[0]-indat[0,index])*3600.*cos(!pi/180.*indat[1,index])
  ypos = (indat[1,index]-med_radec[1])*3600.
  
  xdref = fltarr(nstars,xoutsize,/nozero)
  ydref = fltarr(nstars,youtsize,/nozero)
  image = fltarr(xoutsize,youtsize,nlayers,/nozero)  

; find conversion from gather map pixels to input mosaic pixels
  xpix = scale*(findgen(xoutsize)-med_xy[0])
  ypix = scale*(findgen(youtsize)-med_xy[1])

  if keyword_set(dist) then begin
	areaconv = areaconv*(dist/206265.)^2
  endif

; find values of wcs
  outwcs = {cd:fltarr(2,2),radec_center:med_radec,pix_center:med_xy}
  outwcs.cd[0,0] = -1.*scale/3600.
  outwcs.cd[1,1] = scale/3600.

  for i = 0,xoutsize-1 do xdref[*,i] = xpos-xpix[i]
  for i = 0,youtsize-1 do ydref[*,i] = ypos-ypix[i]
  xdref = temporary(xdref)^2.
  ydref = temporary(ydref)^2.

  for j = 0,youtsize-1 do begin
    if j mod (youtsize/10) eq 0 then print,strtrim(fix(float(j)/youtsize*10.)*10,2)+'%'
    for i = 0,xoutsize-1 do begin

      distsq = xdref[*,i]+ydref[*,j]
      distr = sqrt(distsq)
      binsize = max(distr)/100.

      new = histogram(distr,binsize=binsize)
      sum = new
      sum[0] = 0
      k = 0l
      while sum[(k-1)>0] lt max(numstars) do begin
       sum[k] = sum[(k-1)>0] + new[k]
       k = k + 1
      endwhile
      w = where(distr lt k * binsize + min(distr))
      srt = sort(distr[w])

      area = distsq[w[srt[numstars-1]]]*(areaconv)

      if NOT keyword_set (cubic) then begin
        image[i,j,*] = numstars/!pi/area
      endif else begin
        image[i,j,*] = numstars/4.*3./!pi/(area^1.5)
      endelse
    endfor
  endfor
 
  if keyword_set(write) then begin
   if size(write,/type) eq 7 then outnm=write else $
      outnm = dialog_pickfile(title='Save gather FITS output as?')
   if outnm ne '' then begin
    mkhdr, hdr, image
    putast, hdr, outwcs.cd, outwcs.pix_center, outwcs.radec_center,1,cd_type=2
    print,'Writing '+outnm
    fits_write,outnm,image,hdr
   endif else print,'Gather FITS save cancelled.'
  endif
 print,'Gather_2MASS Finished!!'
 return,image
end
