;+
;   name:
;        ir_proc.pro
;
;   purpose:
;        a procedure which processes the fits images in a given directory
;    and saves the results in '(subdir)_processed' and makes and saves the
;    skyframes in '(subdir)_skyframes'.
;
;   calling sequence:
;        stelir_proc,blk=blk
;
;   inputs:
;        (NONE) all through gui prompts
;
;   keywords:
;       blk = (integer) set to number of files per file block (default=9)
;             (usually use number of images per map; can be determined from
;             observing log)
;
;   output:
;       a processed image for every data image and the skyframes created
;
;   called programs:
;       fits_datmask.pro
;       merge_mask.pro
;       sxpar.pro
;       sxaddpar.pro
;       sf_flat.pro
;
;   history:
;       written by R. Gutermuth - 2-7-02
;       2-12-02 - switched to standard sxpar.pro and sxaddpar.pro
;                 usage for header info and modification (RAG)
;
;-

pro ir_proc,dirname=dirname,blk=blk

;get dirname if not provided
 if not keyword_set(dirname) then dirname = dialog_pickfile(/directory,$
    title='Select a directory with data images')
 if dirname eq '' then begin
  print,'ERROR in ir_proc.pro: No directory specified!'
  retall
 endif

;get image filenames
 f = findfile(dirname+'*?.?*',count=nf)
 if nf eq 0 then begin
  print,'ERROR in ir_proc.pro: No files of that specification!!'
  return
 endif

;read in images in blocks to make sky frames and process them.
 if not keyword_set(blk) then blk = 9

;make receiving directories
 sls = strmid(dirname,strlen(dirname)-1,1)
 subdir = strsplit(dirname,sls,/extract)
 subdir = subdir[n_elements(subdir)-1]
 subdir = strsplit(subdir,'_',/extract)
 subdir = strjoin(subdir[0:n_elements(subdir)-2])
 resdir=filepath(root_dir=dirname+'..',subdirectory=subdir+'_processed','')
 skydir=filepath(root_dir=dirname+'..',subdirectory=subdir+'_skyframes','')
 file_mkdir,resdir
 file_mkdir,skydir

 i = 0 & last = blk - 1 & flg = 0 & oldblk = blk

 while i lt nf do begin
  for j = 0,last do begin

;read in images and masks
   if i+j lt nf then begin
    print,'Reading in '+f[i+j]+'...'
    fits_datmask,f[i+j],im,hdr,mask

;initialize arrays and pointers
    if i eq 0 and j eq 0 then begin
     sim = size(im)
     ims = make_array(sim[1],sim[2],blk,type=sim[sim[0]+1],/nozero)
     meds = make_array(blk,type=sim[sim[0]+1],/nozero)
     im_masks = ptrarr(blk,/allocate_heap)
     outnames = strarr(blk)
     sh = size(hdr)
     hdrs = strarr(sh[1],blk)
     channel = sxpar(hdr,'CHANNEL')
     filter = sxpar(hdr,'FILTER')
     caldir = filepath(root_dir=dirname+'..',subdir=['..','calib_images'],'')
     flatnm = findfile(caldir+'*_'+filter+'_flat.fit',count=nflt)
     case 1 of
      nflt gt 1: begin
       flatnm = dialog_pickfile(path=caldir,filter='*_'+filter+'_flat.fit',$
          title='Select desired flat field',get_path=chkdir)

;boot 'em if they don't pick from the standard directory cuz it screws
;things up
       if chkdir ne caldir then begin
        print,'ERROR in ir_proc.pro: Picked from non-standard directory!'
        retall
       endif
      end
      nflt eq 1: flatnm = flatnm[0]
      else: flatnm = ''
     endcase

;get or make flat
     if flatnm eq '' then begin
      flt = ptr_new(0)
      *flt = sf_flat(filter=filter)
      flat = ptr_new((*flt).flat) & fmask = ptr_new((*flt).mask)
      ptr_free,flt
     endif else begin
      print,'Reading '+flatnm
      fits_datmask,flatnm,flat,fhdr,fmask
      zero = where(flat eq 0,zct)
      if zct gt 0 then flat[zero] = 1.
      flat = ptr_new(temporary(flat))
      fmask = ptr_new(temporary(fmask))
     endelse
    endif

;make output file name and keep in outnames array
    dl = strlen(f[i+j])
    slsh = rstrpos(f[i+j],sls)
    fnm = strmid(f[i+j],slsh+1,dl-slsh-1)
    outnames[j] = resdir+fnm

;merge image mask with proper flat mask and store
    *im_masks[j] = merge_mask(temporary(mask),*fmask)

;make mask image, then flat image, apply mask, and store it all
    ims[*,*,j] = temporary(im) / *flat
    ims[*im_masks[j]+j*sim[1]*sim[2]] = -1000.
    meds[j] = median(ims[*,*,j])
    hdrs[*,j] = temporary(hdr)

;check for incomplete number of images at the end and deal with it
   endif else begin
    ims = ims[*,*,0:j-1] & oldblk = blk & blk = j 
   endelse
  endfor

;offset all images to make their median equal that of the cube
  med = median(ims)

  for k = 0,blk-1 do begin
   ims[*,*,k] = ims[*,*,k] - (meds[k] - med)

;find common mask of the images to go into the skyframe
   if k eq 0 then sf_mask = *im_masks[0] else $
      sf_mask = merge_mask(temporary(sf_mask),*im_masks[k],/agree)
  endfor

;make the skyframe and write in 'skyframes'
  sky = median_stack(ims)
  medsky = median(sky)
  sxaddpar,hdr,'HISTORY','flatted sky frame; median of '+strtrim(blk,2)+$
     ' images'
  print,'Writing '+skydir+'skyframe'+strtrim(fix(i/oldblk),2)+'.fit'
  fits_datmask,/write,skydir+'skyframe'+strtrim(fix(i/oldblk),2)+'.fit',$
     sky,hdr,sf_mask

;subtract skyframes, add median of skyframe, save final images in 'processed'
  for j = 0,blk-1 do begin
   im = ims[*,*,j] - sky - (med - meds[j] - medsky)
   sxaddpar,hdr,'HISTORY','sky frame subtracted; flat fielded'
   print,'Writing '+outnames[j]+' ...'
   fits_datmask,/write,outnames[j],im,hdr,im_mask
  endfor

;switch to next block of files
  i = i + blk
 endwhile

;free the rest of the pointers and end
 ptr_free,im_masks,flat,fmask
 print,'Sky frame subtraction and flat fielding completed!'

end

