;+
;
; rad_mag_hist2 - makes a radially partitioned set of Magnitude histograms 
;                 from a PVDAT file.
;
; CALLING SEQUENCE:
;
;  rad_mag_hist1,radpix,infile=infile,bin_size=bin_size,maxdist=maxdist,
;                center=center,maglimit=maglimit,title=title,xtitle=xtitle,
;                ytitle=ytitle,charsize=charsize
;
;-

pro rad_mag_hist2,radpix,bx1,bx2,by1,by2,$
                  infile=infile,bin_size=bin_size,maxdist=maxdist,$
                  center=center,maglimit=maglimit,$
                  title=title,xtitle=xtitle,ytitle=ytitle,charsize=charsize

 if not keyword_set(infile) then begin
  infile = dialog_pickfile(title='Select PVDAT file',/must_exist)
  if infile eq '' then begin
   print,'Cancelled...'
   return
  endif
 endif

 restore,infile

 if not keyword_set(maglimit) then maglimit = 25.

 w = where(pv_dat[7,*] lt maglimit)
 pv_dat = pv_dat[*,w]

 if not keyword_set(bin_size) then bin_size = .5

 if not keyword_set(center) then begin
  center = [median(pv_dat[0,*]),median(pv_dat[1,*])]
 endif

 dist = sqrt((pv_dat[0,*]-center[0])^2 + (pv_dat[1,*]-center[1])^2)

 if not keyword_set(maxdist) then begin
  if keyword_set(useall) then maxdist = max(dist)+1 else $
     maxdist = min([[max(pv_dat[0,*]),max(pv_dat[1,*])]-center,center])
 endif

 w_in = where(dist lt radpix)
; w_out = where(dist ge radpix and dist lt maxdist) 
 w_out = where(pv_dat[0,*] gt bx1 and pv_dat[0,*] lt bx2 and $
               pv_dat[1,*] gt by1 and pv_dat[1,*] lt by2)
 
 in_hist = [0,histogram([0,reform(pv_dat[7,w_in])],bin=bin_size),0]
 n_in = n_elements(in_hist)
 in_hist[1] = in_hist[1]-1

 out_hist = [0,histogram([0,reform(pv_dat[7,w_out])],bin=bin_size),0]
 n_out = n_elements(out_hist)
 out_hist[1] = out_hist[1]-1

 nbins = max([n_in,n_out])
 if n_in gt n_out then begin
  tmp = out_hist
  out_hist = lonarr(n_in)
  n_out = n_in
  out_hist[0] = tmp
 endif
 if n_out gt n_in then begin
  tmp = in_hist
  in_hist = lonarr(n_out)
  n_in = n_out
  in_hist[0] = tmp
 endif

 bins = (findgen(nbins)-.5) * bin_size

 wh = [(where(in_hist gt 0)),(where(out_hist gt 0))]
 minplot = min(wh)
 if minplot ge 0 then minplot = bins[minplot]-.5*bin_size else minplot = bins[0]+.5*bin_size
 maxplot = max(wh)
 if maxplot ge 0 then maxplot = bins[maxplot]+.5*bin_size else maxplot = max(bins)-.5*bin_size

;in_area = !pi * long(radpix)^2
 in_area = !pi * radpix^2.
; if keyword_set(useall) then out_area=(max(pv_dat[0,*])-min(pv_dat[0,*])) * $
;    (max(pv_dat[1,*]) - min(pv_dat[1,*])) - in_area else $
;    out_area = !pi * maxdist^2 - in_area
    out_area = (bx2-bx1+1.) * (by2-by1+1.)
; print,radpix,maxdist,in_area,out_area

 out_hist = temporary(out_hist) * in_area/out_area

 xrange = [minplot,maxplot]
 yrange = [0,max([in_hist,out_hist])]

 old_pmulti = !p.multi
 !p.multi = [0,1,3]

 if not keyword_set(title) then usetitle = '' else usetitle = title+': '

 numstars = strtrim(fix(total(in_hist)+.5),2)
 numstars2 = strtrim(fix(total(out_hist)+.5),2)

 plot,bins,in_hist,psym=10,xrange=xrange,yrange=yrange,/xstyle,$
    title=usetitle+'Raw Source; '+numstars+' sources',xtitle=xtitle,ytitle=ytitle,charsize=charsize
 plot,bins,out_hist,psym=10,xrange=xrange,yrange=yrange,/xstyle,$
    title=usetitle+'Derived Background Contamination; '+numstars2+' sources',xtitle=xtitle,ytitle=ytitle,charsize=charsize

 fun_hist = in_hist - out_hist
 numstars = strtrim(fix(total(fun_hist)+.5),2)
 numstars2 = strtrim(fix(total(fun_hist > 0)+.5),2)

 plot,bins,fun_hist,psym=10,xrange=xrange,yrange=yrange,/xstyle,$
    title=usetitle+'Source minus Background; '+numstars+' ('+numstars2+') sources',xtitle=xtitle,ytitle=ytitle,charsize=charsize

 !p.multi = old_pmulti

end
