;+
;   NAME:
;     IRAC_LIN
;
;   PURPOSE:
;     This procedure reads in a list of images and linearizes them,
;     outputting the linearized images to directory outdir.
;     (IRAC Specific?)
;
;   CALLING SEQUENCE:
;         IRAC_LIN
;
;   INPUTS:
;         (NONE) all inputs are GUI-based
;
;   KEYWORDS:
;       LINDAT = string containing filename to restore structure from
;                linfind.pro or the structure itself (GUI prompt if not
;                specified)
;       INFILES = string array of FITS filenames to be linearized
;                 (GUI prompt if not specified)
;       OUTDIR = string containing directory to put resulting images
;                (GUI prompt if not specified)
;       USELOG = boolean (gives GUI prompt) or filename of lin log
;                to (re)run
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;
;   OUTPUTS:
;         the original images listed in the designated directory are modified
;         and saved to directory outdir which program will
;         create if it doesn't exist using file_mkdir procedure.
;
;   CALLED PROCEDURES:
;         fits_read, fits_datmask, sxpar(), sxaddpar(), logcheck()
;
;   MODIFICATION HISTORY:
;      6-17-02: modified version of lin.pro for IRAC data (RAG)
;
;-


pro irac_lin,lindat=lindat,outdir=outdir,infiles=infiles,$
             uselog=uselog,savelog=savelog,no_savelog=no_savelog

 if strlowcase(!version.os_family) eq 'windows' then sls = '\' else sls = '/'

;****** begin user input section

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1
 endif

;get lindat if not handed it
 needlin = [1,1]
 if keyword_set(lindat) then begin
  case size(lindat,/type) of
   7: begin
    if file_test(lindat) then needlin[0] = 0 else begin
     print,'ERROR in irac_lin.pro: File '+lindat+' does not exist!'
    endelse
   end
   8: begin
    tags = tag_names(lindat)
    w = where(tags eq 'a' or tags eq 'b' or tags eq 'c' or tags eq 'sat' $
              or tags eq 'mask',nt)
    if nt eq 5 then needlin = [0,0] else begin
     print,'ERROR in irac_lin.pro: lindat specified is invalid!
    endelse
   end
   else: print,'ERROR in irac_lin.pro: lindat specified is invalid!
  endcase
 endif

;get lindat filename if not supplied correctly
 if needlin[0] then begin
  lindat = dialog_pickfile(/must_exist,$
     title='Select file with LINDAT structure')
  if lindat eq '' then begin
   print,'ERROR in irac_lin.pro: No LINDAT structure!'
   retall
  endif
 endif

;get main image names if not supplied
 if not keyword_set(infiles) then infiles=dialog_pickfile(/multiple,$
    /must_exist,title='Select FITS images to linearize')
 nf = n_elements(infiles)
 if nf le 0 or infiles[0] eq '' then begin
  print,'ERROR in irac_lin.pro: No files selected!'
  retall
 endif

;get outdir if not provided
 if not keyword_set(outdir) then outdir = dialog_pickfile(/directory,$
    title='Select a directory for output FITS files')
 if outdir eq '' then begin
  print,'ERROR in irac_lin.pro: No output directory specified!'
  retall
 endif
 if not file_test(strmid(outdir,0,strlen(outdir)-1),/directory) then file_mkdir,outdir

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

;write log if it's called for
 if size(lindat,/type) eq 7 then begin
  if loghuh then save,filename=savelog,outdir,infiles,lindat
 endif else begin
  if loghuh then save,filename=savelog,outdir,infiles
 endelse

;****** end user input section

;read in lindat if not supplied
 if needlin[1] then restore,lindat

;read in all the fits images in the list, compute corrected signal and
;write to a new file in outdir with mask vector as FITS extension

 for i=0,nf-1 do begin
  print,'Reading in '+infiles[i]+' ...'
  fits_read,infiles[i],data,header
  full = strsplit(infiles[i],sls,/extract)
  fname = full[n_elements(full)-1]

;**********stelircam specific stuff

;  coadds = fix(sxpar(header,'CO_ADDS'))
;  data = data/float(coadds[0])

;**********end stelircam specific stuff

;mask out bad pixels
  data = data*lindat.mask

;linearize image
  t = data*0.
  sprime = data*0.
  newa = lindat.a - data
  radical = lindat.b^2 - 4*newa*lindat.c
;  index = where(radical ge 0)
  index = where(radical ge 0 and lindat.c ne 0)
  t[index] = (-lindat.b[index] + sqrt(radical[index])) / (2*lindat.c[index])
  sprime[index] = (lindat.a[index]+lindat.b[index]*t[index]);*(float(coadds[0]))

  index = where(lindat.c eq 0,nz)
  if nz gt 0 then sprime[index] = data[index]

  indexbad = where(radical lt 0,nib)
  if (indexbad[0] gt -1) then sprime[indexbad] = -1000
  print,strtrim(nib,2)+' extra bad pixels found...'

;merge indexbad with lindat.mask
  mask = where(radical lt 0 or lindat.mask eq 0)

;update header
  sxaddpar,header,'HISTORY','linearized; badpix xtension written'

;write linearized image to outdir
  print,'Writing '+outdir+fname+' with badpix vector as FITS xtension...'

;Used fits_datmask to write both image and badpix vector as extension
; (fits_datmask converts mask image to vector of bad pixels before
; writing if it needs to)
  fits_datmask,/write,outdir+fname,sprime,header,mask

 endfor

 print,'IRAC_LIN(-earization) run completed!'

end


