;+
;    NAME:
;       IRAC_LINFIND
;
;    PURPOSE:
;       This function calculates the linearity correction for a series of
;       images, returning the result as an anonymous structure.  (IRAC
;       specific?)
;
;    CALLING SEQUENCE:
;       result = IRAC_LINFIND()
;
;    INPUTS:
;       NONE (GUI-based inputs)
;
;    KEYWORDS:
;       INFILES = string array of FITS filenames of linearity sequence
;                 (GUI prompt if not specified)
;       OUTNAME = string containing filename to save LINDAT structure to
;                 (GUI prompt if not specified, can disable by specifying 'NA')
;       USELOG = boolean (gives GUI prompt) or filename of linfind log
;                to (re)run
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;       KSAT = floating constant saturation level (default is dynamic
;              saturation setting pixel by pixel)
;       TOL = difference tolerance for dynamic saturation
;              setting (default tol=200)
;
;    OUTPUTS:
;       the output is an anonymous structure of the format:
;        a: zeroth order coefficient image
;        b: first order coefficient image
;        c: second order coefficient image
;        mask: all pixels where the .75 > b/median(b) > 1.5
;              are set to zero
;        sat: an array set to the saturation values for each pixel
;       which also is written to disk using IDL save procedure.
;
;
;    NOTE:
;       the fits of several different pixels are plotted as a check.
;
;    CALLED PROCEDURES:
;       fits_read, sxpar(), jail(), dysat(), logcheck()
;
;    HISTORY:
;       6-17-02: modified version of linfind.pro for IRAC data (RAG)
;
;-


function irac_linfind,outname=outname,infiles=infiles,ksat=ksat,tol=tol,$
                      uselog=uselog,no_savelog=no_savelog,savelog=savelog

;****** begin user input section

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1
 endif

;get main image names if not supplied
 if not keyword_set(infiles) then infiles=dialog_pickfile(/multiple,$
    /must_exist,title='Select linearity sequence FITS files')
 nf = n_elements(infiles)
 if nf le 0 or infiles[0] eq '' then begin
  print,'ERROR in irac_linfind.pro: No files selected!'
  retall
 endif

;get outname if not provided
 if not keyword_set(outname) then begin
  huh = 'No'
  while huh ne 'Yes' do begin
   outname = dialog_pickfile(title='Save result as?')
   if file_test(outname) then huh = dialog_message(/question,'File '+outname+$
      'exists...  Overwrite?') else huh = 'Yes'
   if outname eq '' then begin
    print,'Cancel: Not writing result to disk...'
    huh = 'Yes'
   endif
  endwhile
 endif

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

 if NOT keyword_set(ksat) then begin
  sat=1.e6 & ksat = 0
 endif
 if not keyword_set(tol) then tol=200

;write log if it's called for
 if loghuh then save,filename=savelog,outname,infiles,ksat,tol

;****** end user input section

;load in data, (normalized by coadds for STELIRCam), and create
;   array of itimes.

 for i=0,nf-1 do begin
  print,'Reading '+infiles[i]
  fits_read,infiles[i],data,hdr

  if i eq 0 then begin
   sz = size(data)
   datacube = fltarr(nf,sz[1],sz[2],/nozero)
   time = fltarr(nf)
   a = fltarr(sz[1],sz[2])
   b = a & c = a & satmap = a
  endif

  itime = float(sxpar(hdr,'ITIME')/1000.)

  datacube[i,*,*] = float(data)
  time[i] = itime

 endfor

 x = time
 srt = sort(x)
 newx = x[srt]

 old_pmulti= !p.multi
 !p.multi = [0,4,4]

 for j=0,sz[2]-1 do begin
  for i=0,sz[1]-1 do begin

   y = datacube[*,i,j]

;mod-RAG- using a dynamic saturation setting based on slope between
;         successive data points  (through dysat.pro)
   if not keyword_set(ksat) then begin
    newy = y[srt]
    chop = dysat(newx,newy,tol)
    if chop eq max(x) then sat = 1.e6 else $
       sat = mean(newy[where(x eq chop)]) > 0
   endif else sat = ksat

;chop saturated data
   index = where(x le chop,ns)
   if (ns lt 5) then begin
    y = fltarr(nf)*0.
    index = where(y le sat)
   endif

;do second order polynomial fit
   result = poly_fit(x[index],y[index],2)
   a[i,j] = result[0]
   b[i,j] = result[1]
   c[i,j] = result[2]
   satmap[i,j] = sat

   if i eq 0 then print,'Beginning row '+strtrim(j,2)
;show representative plots
   if (total(i eq [4,5,6,7]) + total(j eq [60,120,180,240]) ge 2) then begin
    plot,x,(y-(a[i,j]+b[i,j]*x+c[i,j]*x*x))/y,psym=2,yrange=[-0.20,0.20]
;    oplot,x,x*0.+0.01
;    oplot,x,x*0.-0.01
    plot,x,y,psym=2
    oplot,x,a[i,j]+b[i,j]*x+c[i,j]*x*x
   endif
  endfor
 endfor

;make mask image
 mask = a*0+1
; index = where(abs(b-median(b)) gt 5.*stdev(b))
 index = where(b/median(b) gt 1.5 or b/median(b) lt .75,nbad)
 if nbad gt 0 then mask[index] = 0

;return output structure
 lindat = {a:a,b:b,c:c,mask:mask,sat:satmap}

;write results with IDL save procedure
 if outname ne '' and outname ne 'NA' then begin
  print,'Saving lindat structure...'
  save,filename=outname,lindat
 endif else print,'Not saving LINDAT structure.'

;cleanup and return results
 !p.multi = old_pmulti
 print,'IRAC_LINFIND run completed!'
 return,lindat

end


