;+
;   NAME:
;     LIN
;
;   PURPOSE:
;     This procedure reads in a list of images and linearizes them,
;     outputting the linearized images to directory outdir.
;
;   CALLING SEQUENCE:
;         LIN
;
;   INPUTS:
;         (NONE) all inputs are GUI-based
;
;   KEYWORDS:
;       LINDAT = string containing filename to restore structure from
;                linfind.pro or the structure itself (GUI prompt if not
;                specified)
;       INFILES = string array of FITS filenames to be linearized
;                 (GUI prompt if not specified)
;       OUTDIR = string containing directory to put resulting images
;                (GUI prompt if not specified)
;       USELOG = boolean (gives GUI prompt) or filename of lin log
;                to (re)run
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;
;   OUTPUTS:
;         the original images listed in the designated directory are modified
;         and saved to directory outdir which program will
;         create if it doesn't exist using file_mkdir procedure.
;
;   CALLED PROCEDURES:
;         fits_read, fits_datmask, sxpar(), sxaddpar(), logcheck()
;
;   MODIFICATION HISTORY:
;    written by Dawn Peterson - 7/19/00
;    modified by Rob Gutermuth - 5-25-01
;    modified by Rob Gutermuth - 5-30-01
;    8-24-01 - cleaned out all the commented code, as well as made conversion
;              to fits_datmask.pro for writing processed images and keeping
;              track of bad pixel mask for each image.  Also made changes to
;              increase platform independence. Should work for Windows and UNIX
;              at the bare minimum.  (RAG)
;    1-24-02 - ripped this apart and killed/isolated all stelircam specific
;              data sorting, among other things...  (RAG)
;    2-4-02 - made some more mods to implement reorganization (RAG)
;    2-12-02 - switched to standard sxpar.pro and sxaddpar.pro for
;              header info and manipulation (RAG)
;    5-15-02 - ripped out fascist file management to help modularize a bit.
;              Also added logging and log restoration as well as expectation
;              of linearity results as a structure. (RAG)
;    6-19-02 - fixed minor/unlikely bug of zero-valued c coefficient, which we
;              divide by in quadratic formula (it happened!!) (RAG)
;    7-9-02 - eliminated STELIRCam specificness...  now technically can 
;             linearize data from any instrument (Ha!  We'll see about that!)
;             (RAG)
;    3-13-03 - added date/time logging to HISTORY comments added to FITS header
;              (RAG)
;
;-


pro lin,lindat=lindat,outdir=outdir,infiles=infiles,$
        uselog=uselog,savelog=savelog,no_savelog=no_savelog

 if strlowcase(!version.os_family) eq 'windows' then sls = '\' else sls = '/'

;****** begin user input section

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1
 endif

;get lindat if not handed it
 needlin = [1,1]
 if keyword_set(lindat) then begin
  case size(lindat,/type) of
   7: begin
    if file_test(lindat) then needlin[0] = 0 else begin
     print,'ERROR in lin.pro: File '+lindat+' does not exist!'
    endelse
   end
   8: begin
    tags = tag_names(lindat)
    w = where(tags eq 'a' or tags eq 'b' or tags eq 'c' or tags eq 'sat' $
              or tags eq 'mask',nt)
    if nt eq 5 then needlin = [0,0] else begin
     print,'ERROR in lin.pro: lindat specified is invalid!
    endelse
   end
   else: print,'ERROR in lin.pro: lindat specified is invalid!
  endcase
 endif

;get lindat filename if not supplied correctly
 if needlin[0] then begin
  lindat = dialog_pickfile(/must_exist,$
     title='Select file with LINDAT structure')
  if lindat eq '' then begin
   print,'ERROR in lin.pro: No LINDAT structure!'
   retall
  endif
 endif

;get main image names if not supplied
 if not keyword_set(infiles) then infiles=dialog_pickfile(/multiple,$
    /must_exist,title='Select FITS images to linearize')
 nf = n_elements(infiles)
 if nf le 0 or infiles[0] eq '' then begin
  print,'ERROR in lin.pro: No files selected!'
  retall
 endif

;get outdir if not provided
 if not keyword_set(outdir) then outdir = dialog_pickfile(/directory,$
    title='Select a directory for output FITS files')
 if outdir eq '' then begin
  print,'ERROR in lin.pro: No output directory specified!'
  retall
 endif
 if not file_test(strmid(outdir,0,strlen(outdir)-1),/directory) then file_mkdir,outdir

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

;write log if it's called for
 if size(lindat,/type) eq 7 then begin
  if loghuh then save,filename=savelog,outdir,infiles,lindat
 endif else begin
  if loghuh then save,filename=savelog,outdir,infiles
 endelse

;****** end user input section

;read in lindat if not supplied
 if needlin[1] then restore,lindat

;read in all the fits images in the list, compute corrected signal and
;write to a new file in outdir with mask vector as FITS extension

 for i=0,nf-1 do begin
  print,'Reading in '+infiles[i]+' ...'
  fits_read,infiles[i],data,header
  full = strsplit(infiles[i],sls,/extract)
  fname = full[n_elements(full)-1]

  ins = strtrim(sxpar(header,'INSTRUME'),2)

;**********stelircam specific stuff
  if ins eq 'ADS SBRC Camera' then begin
   coadds = fix(sxpar(header,'CO_ADDS'))
   data = temporary(data)/float(coadds[0])
  endif
;**********end stelircam specific stuff

;mask out bad pixels
  data = temporary(data)*lindat.mask

;linearize image
  t = data*0.
  sprime = data*0.
  newa = lindat.a - data
  radical = lindat.b^2 - 4*newa*lindat.c
;  index = where(radical ge 0)
  index = where(radical ge 0 and lindat.c ne 0)
  t[index] = (-lindat.b[index] + sqrt(radical[index])) / (2*lindat.c[index])

;**********stelircam specific stuff
  if ins eq 'ADS SBRC Camera' then begin
   sprime[index] = (lindat.a[index]+lindat.b[index]*t[index])*(float(coadds[0]))
;**********end stelircam specific stuff

  endif else begin
   sprime[index] = (lindat.a[index]+lindat.b[index]*t[index])
  endelse

  index = where(lindat.c eq 0,nz)
  if nz gt 0 then sprime[index] = data[index]

  indexbad = where(radical lt 0,nib)
  if (indexbad[0] gt -1) then sprime[indexbad] = -1000
  print,strtrim(nib,2)+' extra bad pixels found...'

;merge indexbad with lindat.mask
  mask = where(radical lt 0 or lindat.mask eq 0)

;update header
  systm = systime(/utc)
  sxaddpar,header,'HISTORY',systm+' UTC: linearized'
  sxaddpar,header,'HISTORY',systm+' UTC: badpix vector extension written'

;write linearized image to outdir
  print,'Writing '+outdir+fname+' with badpix vector as FITS extension...'

;Used fits_datmask to write both image and badpix vector as extension
; (fits_datmask converts mask image to vector of bad pixels before
; writing if it needs to)
  fits_datmask,/write,outdir+fname,sprime,header,mask

 endfor

 print,'LIN(-earization) run completed!'

end


