;----------------------------------------------------------------
;
;   This program overlays a CO data cube onto an image
;
;----------------------------------------------------------------

pro overlay,image,overlay,min=min,max=max,$
  l0=l0,b0=b0,dl=dl,db=db,vel1=vel1,vel2=vel2,moment=moment,$
  contour=contour,plotpos=plotpos,order=order,percent=percent,$
  ps=ps
  
  if NOT keyword_set(order) then order = "vxy"
  if NOT keyword_set(percent) then percent = 0

; read in image and convert to bytscale 
  fits_read,image,image,header
  fits_read,overlay,cube,header_over

; determine pixels of image
  fitsaxes,header,x,y,z
  nx = n_elements(x)
  ny = n_elements(y)

; determine velocity channel spacing
  fitsaxes,header_over,x_over,y_over,z_over,order=order

; integrate cube over velocity range of perseus arm
  sum = int_slice(cube,z_over,vel1=vel1,vel2=vel2,$
        order=order)
  
; find lower left hand and upper right hand pixels.
  pixll = $
   [fix((l0+dl/2.-x[0])/(x[1]-x[0])),fix((b0-db/2.-y[0])/(y[1]-y[0]))] 
  pixll = pixll > [0,0]  & pixll = pixll < [nx-1,nx-1]
  pixur = $
   [fix((l0-dl/2.-x[0])/(x[1]-x[0])),fix((b0+db/2.-y[0])/(y[1]-y[0]))] 
  pixur = pixur > [0,0]  & pixur = pixur < [ny-1,ny-1]
  siz = pixur-pixll+[1,1]
  print,pixll,pixur
  xnew = x[pixll[0]:pixur[0]] 
  ynew = y[pixll[1]:pixur[1]]

; create byte array of data with specified size
  image = alog(image[pixll[0]:pixur[0],pixll[1]:pixur[1]])
  bimage = bytscl(image,min=min,max=max,top=!d.n_colors-1)

; create interpolation arrays and interpolate
  xinterp = interp_pix(xnew,x_over)
  yinterp = interp_pix(ynew,y_over)
  print,"first max sum =",max(sum)
  sum = interpolate(sum,xinterp,yinterp,/grid,missing=0)
  print,"new max sum =",max(sum)


; set contour levels
  if not keyword_set(contour) then begin
    contour = [.3,.4,.5,.6,.7,.8,.9,1.0]*max(sum)
  endif else begin
    if percent eq 1 then contour=contour*max(sum)/100. 
  endelse

; set up postscript if plot
   if keyword_set(ps) then begin
     dpcm=1000.                      ; dots/cm
     xpaper=21.6                     ; xsize of paper (cm)
     ypaper=27.9                     ; ysize of paper (cm)
     xmargin=2.5                     ; x-border (cm)
     ymargin=2.5                     ; y-border (cm)

     xdisp = 20
     ydisp = 20
     aspect = db/dl
     disp_asp = 1.
     xoffset=(ypaper-xdisp)/2.0              ; xoffset: center
     yoffset=(xpaper-ydisp*aspect)/2.0  
     set_plot,'ps',/copy
     device,/portrait,xoff=xoffset/4.,yoff=yoffset*2.,xsize=xpaper,ysize=ypaper
     device,/color,bits=8
     plot,x,y,xsty=5,ysty=5,/noerase,/nodata
;      loadct,34
;      loadct,3
       @loadcolor
     print,!d.table_size
   endif

; set size of plotting window
  x=findgen(256) & y=findgen(256)
  if keyword_set(plotpos) then begin
    n = 1
    plot,x,y,xsty=5,ysty=5,/nodata,/noerase,pos=plotpos,/device
    px=!x.window*!d.x_vsize
    py=!y.window*!d.y_vsize
    xsize = px[1]-px[0] & ysize=py[1]-py[0]
  endif else begin
    n = fix(1.*min([!d.x_vsize,!d.y_vsize])/max(siz))
    plot,x,y,/nodata,xstyle=5,ystyle=5
    if NOT keyword_set(ps) then begin
      px=!x.window*!d.x_vsize
      py=!y.window*!d.y_vsize
      xsize = n*siz[0]-1
      ysize = n*siz[1]-1
      plotpos = replicate(0,4)
      plotpos[0] = px[0]
      plotpos[1] = py[0]
      plotpos[2] = px[0]+xsize
      plotpos[3] = py[0]+ysize
    endif else begin
      xsize = xdisp
      ysize = ydisp*siz[1]/siz[0]
      help,xsize,ysize
      plotpos = replicate(0,4)
      plotpos[0] = xmargin
      plotpos[1] = ymargin
      plotpos[2] = plotpos[0]+xsize
      plotpos[3] = plotpos[1]+ysize
    endelse
   endelse

; plot image and then overlay contours
   tv,rebin(bimage,n*siz[0],n*siz[1],/sample),plotpos[0],plotpos[1
],$
       xsize=xsize, ysize=ysize
   contour,sum,xnew,ynew,xstyle=1,ystyle=1,c_color=[255],$
       position=plotpos, $
       levels=contour, xrange=[max(xnew),min(xnew)],$
       /device,/noerase
   loadct,3

; if making postscript file, close device and change plot back to 'x'
   if keyword_set(ps) and NOT keyword_set(continue) then begin
      device,/close
      set_plot,'x',/copy
   endif

return
end

