; fitaxes
;
; creates x, y, and z axis from a fits header.   This program
; loads the x[i],y[j], and z[k] vector with the coordinates
; of the pixel ijk.  These coordinates are determined from the
; standard header variables: CRVAL, CRPIX and CRDELT.  Normally,
; the 0,0 pixel location is set to be the lower, left hand corner
; of the lower, left hand pixel.  If the IRAF keyword is set, then
; the IRAF convention is used, and it is the 0,0 position is the
; center of the lower, left hand pixel.
; 
;
; parameters: header: name of string array containing header
;             x:      vector for x axis
;             y:      vector of y axis
;             z:      vector of z axis
;             blank:  blanking value
;             quiet:  prints no information
;             iraf:   uses IRAF convention (coordinates refer to pixel
;                        centers)
;             order:  can select the order of the axis - possible values
;                     are "xy","vxy","xyv","xvy"
;
; Author: Dap Hartmann
; Creation Date:  25 Jan 1995
; Last Modified by Tom Megeath:  14 Sep 1998
;
pro fitsaxes,header,x,y,z,blank=blank,quiet=quiet,iraf=iraf,order=order,$
             cosy=cosy



; set to default value of order if keyword is not set
if NOT keyword_set(order) then order = "vxy"

; set default values of header variables
naxis1=0 & naxis2=0 & naxis3=0
crval1=0.0 & crvax2=0.0 & crval3=0.0
crpix1=0.0 & crpix2=0.0 & crpix3=0.0
cdelt1=0.0 & cdelt2=0.0 & cdelt3=0.0
ctype1='????????' & ctype2='????????' & ctype3='????????'
blank=-9999.99 & bscale=1.0 & bzero=0.0

s=size(header) & nlines=s(1)
for i=0,nlines-1 do begin
  if (strpos(header(i),'NAXIS1') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & naxis1=fix(s)
  endif
  if (strpos(header(i),'NAXIS2') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & naxis2=fix(s)
  endif
  if (strpos(header(i),'NAXIS3') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & naxis3=fix(s)
  endif
  if (strpos(header(i),'CRVAL1') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & crval1=double(s)
  endif
  if (strpos(header(i),'CRVAL2') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & crval2=double(s)
  endif
  if (strpos(header(i),'CRVAL3') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & crval3=double(s)
  endif
  if (strpos(header(i),'CRPIX1') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & crpix1=double(s)
  endif
  if (strpos(header(i),'CRPIX2') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & crpix2=double(s)
  endif
  if (strpos(header(i),'CRPIX3') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & crpix3=double(s)
  endif
  if (strpos(header(i),'CDELT1') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & cdelt1=double(s)
  endif
  if (strpos(header(i),'CDELT2') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & cdelt2=double(s)
  endif
  if (strpos(header(i),'CDELT3') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & cdelt3=double(s)
  endif
  if (strpos(header(i),'CTYPE1') eq 0) then begin
    s=strmid(header(i),10,20) & ctype1=strtrim(s)
  endif
  if (strpos(header(i),'CTYPE2') eq 0) then begin
    s=strmid(header(i),10,20) & ctype2=strtrim(s)
  endif
  if (strpos(header(i),'CTYPE3') eq 0) then begin
    s=strmid(header(i),10,20) & ctype3=strtrim(s)
  endif
  if (strpos(header(i),'BLANK') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & blank=double(s)
  endif
  if (strpos(header(i),'BSCALE') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & bscale=double(s)
  endif
  if (strpos(header(i),'BZERO') eq 0) then begin
    s=strmid(header(i),10,20) & s=strtrim(s) & bzero=double(s)
  endif
end

if (naxis1 gt 0) then begin
if keyword_set(iraf) then begin
  a=crval1+cdelt1*(findgen(naxis1)-crpix1+1)-cdelt1/2.
endif else begin
  a=crval1+cdelt1*(findgen(naxis1)-crpix1+1)
endelse
  acenter = crval1
  if NOT keyword_set(quiet) then print,format='("1st Axis: ",A10,F15.4," ==> ",F12.4,"  by",F11.5,"   (",I0,")")',ctype1,a(0),a(naxis1-1),cdelt1,naxis1
endif else a=0

if (naxis2 gt 0) then begin
if keyword_set(iraf) then begin
  b=crval2+cdelt2*(findgen(naxis2)-crpix2+1)-cdelt2/2.
endif else begin
  b=crval2+cdelt2*(findgen(naxis2)-crpix2+1)
endelse
  bcenter = crval2
  if NOT keyword_set(quiet) then print,format='("2nd Axis: ",A10,F15.4," ==> ",F12.4,"  by",F11.5,"   (",I0,")")',ctype2,b(0),b(naxis2-1),cdelt2,naxis2
endif else b=0

if (naxis3 gt 0) then begin
if keyword_set(iraf) then begin
  c=crval3+cdelt3*(findgen(naxis3)-crpix3+1)-cdelt3/2.
endif else begin
  c=crval3+cdelt3*(findgen(naxis3)-crpix3+1)
endelse
  if NOT keyword_set(quiet) then print,format='("3rd Axis: ",A10,F15.4," ==> ",F12.4,"  by",F11.5,"   (",I0,")")',ctype3,c(0),c(naxis3-1),cdelt3,naxis3
endif else c=0

blank=blank+bscale+bzero

if (naxis3 le 1 and order ne "ra") then order = "xy"

  case order of
    "vxy": begin 
            z = a
            x = b 
            xc = bcenter
            y = c 
           end
    "xyv": begin 
            x = a
            xc = acenter
            y = b
            z = c 
           end
    "xvy": begin 
            x = a
            xc = acenter
            z = b 
            y = c 
           end
    "xy" : begin 
            x = a
            xc = acenter
            y = b 
           end
  endcase
  if keyword_set(cosy) then begin
     midy = (max(y)-min(y))/2.+min(y)  
     x = (x-xc)/cos(midy*2.*3.141592654/360.)+xc
  endif

return
end



