;+
;   name:
;       mosaic_two.pro
;
;   purpose:
;       mosaicing subroutine for interactive mosaic construction
;
;   calling sequence:
;       mosaic_two, mosaic, mosmask, data, datamask, xoff, yoff,
;       xmoff, ymoff, kx, ky, autoffset=autoffset
;
;   inputs:
;       mosaic = mosaic image which should be modified
;       mosmask = mask accompanying mosaic data
;       data = data array to be added to mosaic
;	datamask = mask for data
;       xoff,yoff = x and y offsets of data-these values will
;				 be modified and returned by mosaic_two
;       xmoff,ymoff = x and y offsets of initial mosaic - these
;				 values are returned so that the offsets
;				 of the previously mosaicked images
;				 can be registered.
;       kx,ky = polywarp matrices, tweaked with refined offsets
;       autoffset:   set on to automatically adjust median levels
;				 between frames
;   output:
;       result is an image mosaic
;
;   called programs:
;       none
; 
;   history:
;       7-20-00  - first version released, written by S. T. Megeath
;       3-20-01  - added autoffset keyword (STM)
;       4-11-01  - (STM and DEP) fixed lines 74, 75 and 76 by using values
;                 of .99 instead of .9 and .01 instead of .1 as conditions.
;                 before the change, some images weren't aligning correctly
;                 because they fell under the .1 threshold.  added lines
;                 77-78.
;       4-23-01  - DEP took out lines 77-78 and added an else
;                  statement to the if in lines 75 and 76.
;       6-21-01  - changed function of autoffset to offset by difference
;                  in median values of pre-expansion mosaic and image being
;                  added.  (RAG)
;       7-17-01  - changed some of the calcs to use the temporary function
;                  to use memory more efficiently (RAG)
;       1-16-04 - massively bastardized from mosaic_two.pro for IRAC data. (RAG)
;
;-

pro irac_ccrefine,mosaic,mosmask,data,datamask,xoff,yoff,kx,ky

  orig = [xoff,yoff]
  irac_spoffset,data,hdr,datamask,kx,ky,xoff,yoff

  imtmp = data*datamask
  imtmp = poly_2d(temporary(imtmp),kx,ky,1,missing=0)
  masktmp = poly_2d(datamask,kx,ky,1,missing=0)

; find array sizes
  xmos = n_elements(mosaic[*,0])
  ymos = n_elements(mosaic[0,*])
  xim  = n_elements(data[*,0])+2
  yim  = n_elements(data[0,*])+2

; expand image
  im = replicate(0.,xim,yim)
  mask = replicate(0.,xim,yim)
;  im[1:xim-2,1:yim-2] = data  
;  mask[1:xim-2,1:yim-2] = datamask
  im[1:xim-2,1:yim-2] = temporary(imtmp)
  mask[1:xim-2,1:yim-2] = temporary(masktmp)
  masktmp = mask*0
  tempmask = where(mask gt .1,ngd)
  if ngd gt 0 then begin
   masktmp[tempmask] = 1
   im[tempmask] = im[tempmask]/mask[tempmask]
  endif
  im = im*masktmp
  im = im+(1-masktmp)*median(im[tempmask])
  mask = mask*masktmp
  
  xoff = xoff-1
  yoff = yoff-1.

; find edges of data and necessary mosaic size 
  rmos = xmos > (fix(xoff)+xim+1)
  lmos = 0 < fix(xoff)-1
  umos = ymos > (fix(yoff)+yim+1)
  dmos = 0 < fix(yoff)-1
  mosxsiz = rmos-lmos+1
  mosysiz = umos-dmos+1
;  print,'mosaic edges',xmos,ymos,fix(yoff)+yim+1,rmos,lmos,umos,dmos

; create newmosaic and load mosaic with original mosaic
  newmosaic = replicate(0.0,mosxsiz,mosysiz)
  newmosmask = replicate(0.0,mosxsiz,mosysiz)
  mosmasktmp = replicate(0.0,mosxsiz,mosysiz)
  newmosaic[-lmos:xmos-lmos-1,-dmos:ymos-dmos-1] = mosaic
  newmosmask[-lmos:xmos-lmos-1,-dmos:ymos-dmos-1] = mosmask
  tempmask = where(newmosmask gt 0.1,ngd)
  if ngd gt 0 then mosmasktmp[tempmask] = 1
  newmosaic = newmosaic*mosmasktmp
  newmosaic = newmosaic+(1-mosmasktmp)*median(newmosaic[tempmask])

;  help,mosaic
  xmoff = -lmos
  ymoff = -dmos
  xoff = xoff-lmos
  yoff = yoff-dmos

; calculate offsets in integer pixels and fractional pixels
;  print,xoff,yoff
  x0 = fix(xoff+.99) & y0 = fix(yoff+.99) & xlen = xim & ylen = yim
  if (xoff-1.*fix(xoff) gt .01) then xoff = x0*1.-xoff else xoff=0.
  if (yoff-1.*fix(yoff) gt .01) then yoff = y0*1.-yoff else yoff=0.

; cross-correllation coalignment refinement
  cmbomask = masktmp*mosmasktmp[x0:x0+xlen-1,y0:y0+ylen-1]
  wx = where(total(cmbomask,2) gt 0,nxg)
  wy = where(total(cmbomask,1) gt 0,nyg)
  x1 = wx[0] & x2=wx[nxg-1]
  y1 = wy[0] & y2=wy[nyg-1]
;  x1 = wx[0]+fix((wx[nxg-1]-wx[0])*.1) & x2=wx[nxg-1]-fix((wx[nxg-1]-wx[0])*.1)
;  y1 = wy[0]+fix((wy[nyg-1]-wy[0])*.1) & y2=wy[nyg-1]-fix((wy[nyg-1]-wy[0])*.1)
;  ss = xyoff(newmosaic[x0:x0+xlen-1,y0:y0+ylen-1],im,xlen,ylen,1,masktmp*mosmasktmp[x0:x0+xlen-1,y0:y0+ylen-1])
  ss = xyoff(newmosaic[x0+x1:x0+x2,y0+y1:y0+y2]*cmbomask[x1:x2,y1:y2],im[x1:x2,y1:y2]*cmbomask[x1:x2,y1:y2],x2-x1+1,y2-y1+1,1,cmbomask[x1:x2,y1:y2])

;  xoff = x0 + ss[0] + 1. + lmos
;  yoff = y0 + ss[1] + 1. + dmos
  xoff = orig[0] + ss[0]
  yoff = orig[1] + ss[1]

;  irac_spoffset,data,hdr,datamask,kx,ky,xoff,yoff

end
